@@ -2,7 +2,7 @@
 # Relational
 # Copyright (C) 2009  Salvo "LtWorf" Tomaselli
 # 
-# Relation is free software: you can redistribute it and/or modify
+# Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
@@ -67,9 +67,9 @@ def recoursive_scan(function,node,rels=None):
     
 
 def duplicated_select(n):
-    changes=0
     '''This function locates and deletes things like
     σ a ( σ a(C)) and the ones like σ a ( σ b(C))'''
+    changes=0
     if n.name=='σ' and n.child.name=='σ':        
         if n.prop != n.child.prop: #Nested but different, joining them
             n.prop = n.prop + " and " + n.child.prop
@@ -81,10 +81,15 @@ def duplicated_select(n):
 
 def futile_union_intersection_subtraction(n):
     '''This function locates things like r ᑌ r, and replaces them with r.
-    σ k (r) ᑌ r with r
-    σ k (r) ᑎ r with σ k (r)
+    R ᑌ R  --> R 
+    R ᑎ R --> R 
+    R - R --> σ False (R)
+    σ k (R) - R --> σ False (R) 
+    R - σ k (R) --> σ not k (R) 
+    σ k (R) ᑌ R --> R 
+    σ k (R) ᑎ R --> σ k (R)
     '''
-    #TODO document into the wiki
+    
     changes=0
     
     if n.name in ('ᑌ','ᑎ') and n.left==n.right:
@@ -102,13 +107,19 @@ def futile_union_intersection_subtraction(n):
             replace_node(n,n.left)
         else:
             replace_node(n,n.right)
-    #elif (n.name == '-' and ((n.left.name=='σ' and n.left.child==n.right) or (n.right.name=='σ' and n.right.child==n.left))): #Intersection of two equal things, but one has a selection
-    elif n.name=='-' and n.left==n.right:#Empty relation
+    #TODO make work the following line...
+    elif (n.name == '-' and (n.right.name=='σ' and n.right.child==n.left)): #Subtraction of two equal things, but one has a selection
+        n.name=n.right.name
+        n.kind=n.right.kind
+        n.child=n.right.child
+        n.prop='(not (%s))' % n.right.prop
+        n.left=n.right=None
+    elif (n.name=='-' and ((n.left==n.right) or (n.left.name=='σ' and n.left.child==n.right)) ):#Empty relation
         changes=1
         n.kind=optimizer.UNARY
         n.name='σ'
         n.prop='False'
-        n.child=n.left
+        n.child=n.left.get_left_leaf()
         #n.left=n.right=None
 
     return changes+recoursive_scan(futile_union_intersection_subtraction,n)
@@ -175,7 +186,6 @@ def swap_union_renames(n):
     and replaces them with
     ρ a➡b(R ᑌ Q).
     Does the same with subtraction and intersection'''
-    #TODO document into the wiki
     changes=0
     
     if n.name in ('-','ᑌ','ᑎ') and n.left.name==n.right.name and n.left.name=='ρ':
@@ -297,7 +307,7 @@ def tokenize_select(expression):
     selection. The expression can contain parenthesis.
     It will use a subclass of str with the attribute level, which
     will specify the nesting level of the token into parenthesis.'''
-    
+    sel_op=('//=','**=','and ','not ','in ','//','**','<<','>>','==','!=','>=','<=','+=','-=','*=','/=','%=','or ','+','-','*','/','&','|','^','~','<','>','%','=','(',')',',','[',']')
     l=0
     while l!=len(expression):
         l=len(expression)
@@ -316,34 +326,17 @@ def tokenize_select(expression):
         elif expression[0:1]==')':
             level-=1
         
-        if expression[0:3] in sel_op:#3char op
-            t=level_string(temp)
-            t.level=level
-            tokens.append(t)
-            temp=''
-            t=level_string(expression[0:3])
-            t.level=level            
-            tokens.append(t)
-            expression=expression[3:]
-        elif expression[0:2] in sel_op:#2char op
-            t=level_string(temp)
-            t.level=level
-            tokens.append(t)
-            temp=''
-            t=level_string(expression[0:2])
-            t.level=level
-            tokens.append(t)
-            expression=expression[2:]
-        elif expression[0:1] in sel_op:#1char op
-            t=level_string(temp)
-            t.level=level
-            tokens.append(t)
-            temp=''
-            t=level_string(expression[0:1])
-            t.level=level
-            tokens.append(t)
-            expression=expression[1:]
-        elif expression[0:1]=="'":#String
+        for i in range(4,0,-1):#operators
+            if expression[0:i] in sel_op:
+                t=level_string(temp)
+                t.level=level
+                tokens.append(t)
+                temp=''
+                t=level_string(expression[0:i].strip())
+                t.level=level
+                tokens.append(t)
+                expression=expression[i:]
+        if expression[0:1]=="'":#String
             end=expression.index("'",1)
             while expression[end-1]=='\\':
                 end=expression.index("'",end+1)
@@ -374,7 +367,6 @@ def swap_rename_projection(n):
     and more important, will hopefully allow further optimizations.
     Will also eliminate fields in the rename that are cutted in the projection.
     '''
-    #TODO document into the wiki
     changes=0
     
     if n.name=='π' and n.child.name=='ρ':
@@ -551,4 +543,6 @@ def selection_and_product(n,rels):
         
 general_optimizations=[duplicated_select,down_to_unions_subtractions_intersections,duplicated_projection,selection_inside_projection,subsequent_renames,swap_rename_select,futile_union_intersection_subtraction,swap_union_renames,swap_rename_projection]
 specific_optimizations=[selection_and_product]
-        
\ No newline at end of file
+
+if __name__=="__main__":
+    print tokenize_select("skill == 'C' and  id % 2 == 0")
\ No newline at end of file
