@@ -3,7 +3,7 @@
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 # 
-# Relation is free software: you can redistribute it and/or modify
+# Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
@@ -22,10 +22,10 @@
 RELATION=0
 UNARY=1
 BINARY=2
-b_operators=('*','-','ᑌ','ᑎ','ᐅᐊ','ᐅLEFTᐊ','ᐅRIGHTᐊ','ᐅFULLᐊ')
+b_operators=('*','-','ᑌ','ᑎ','÷','ᐅᐊ','ᐅLEFTᐊ','ᐅRIGHTᐊ','ᐅFULLᐊ')
 u_operators=('π','σ','ρ')
 
-op_functions={'*':'product','-':'difference','ᑌ':'union','ᑎ':'intersection','ᐅᐊ':'join','ᐅLEFTᐊ':'outer_left','ᐅRIGHTᐊ':'outer_right','ᐅFULLᐊ':'outer','π':'projection','σ':'selection','ρ':'rename'}
+op_functions={'*':'product','-':'difference','ᑌ':'union','ᑎ':'intersection','÷':'division','ᐅᐊ':'join','ᐅLEFTᐊ':'outer_left','ᐅRIGHTᐊ':'outer_right','ᐅFULLᐊ':'outer','π':'projection','σ':'selection','ρ':'rename'}
 
 class node (object):
     '''This class is a node of a relational expression. Leaves are relations and internal nodes are operations.
@@ -105,6 +105,30 @@ class node (object):
         else:
             return self.name
         pass
+    def printtree(self,level=0):
+        '''Prints a representation of the tree using indentation'''
+        r=''
+        for i in range(level):
+            r+='  '
+        r+=self.name
+        if self.name in b_operators:
+            r+=self.left.printtree(level+1)
+            r+=self.right.printtree(level+1)
+        elif self.name in u_operators:
+            r+='\t%s\n' % self.prop
+            r+=self.child.printtree(level+1)
+            
+        return '\n'+r
+    def get_left_leaf(self):
+        '''This function returns the most left leaf in the tree. It is needed by some optimizations.'''
+        if self.kind==RELATION:
+            return self
+        elif self.kind==UNARY:
+            return self.child.get_left_leaf()
+        elif self.kind==BINARY:
+            return self.left.get_left_leaf()
+        
+        
     def result_format(self,rels):
         '''This function returns a list containing the fields that the resulting relation will have.
         Since it needs to know real instances of relations, it requires a dictionary where keys are
@@ -116,6 +140,8 @@ class node (object):
             return list(rels[self.name].header.attributes)
         elif self.kind==BINARY and self.name in ('-','ᑌ','ᑎ'):
             return self.left.result_format(rels)
+        elif self.kind==BINARY and self.name=='÷':
+            return list(set(self.left.result_format(rels)) - set(self.right.result_format(rels)))
         elif self.name=='π':
             l=[]
             for i in self.prop.split(','):
@@ -123,7 +149,7 @@ class node (object):
             return l
         elif self.name=='*':
             return self.left.result_format(rels)+self.right.result_format(rels)
-        elif self.name=='σ' :
+        elif self.name=='σ':
             return self.child.result_format(rels)
         elif self.name=='ρ':
             _vars={}
@@ -233,7 +259,10 @@ def tokenize(expression):
         elif expression.startswith("ᑎ") or expression.startswith("ᑌ"): #Binary short 3 bytes
             items.append(expression[0:3]) #Adding operator in the top of the list
             expression=expression[3:].strip() #Removing operator from the expression
-
+            state=4
+        elif expression.startswith("÷"): #Binary short 2 bytes
+            items.append(expression[0:2]) #Adding operator in the top of the list
+            expression=expression[2:].strip() #Removing operator from the expression
             state=4
         elif expression.startswith("ᐅ"): #Binary long
             i=expression.find("ᐊ")
