@@ -2,7 +2,7 @@
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 # 
-# Relation is free software: you can redistribute it and/or modify
+# Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
@@ -86,7 +86,7 @@ class relation (object):
                 res+=" ".join(r)
             fp.write(res)
         fp.close() #Closing file
-    def rearrange(self,other):
+    def _rearrange_(self,other):
         '''If two relations share the same attributes in a different order, this method
         will use projection to make them have the same attributes' order.
         It is not exactely related to relational algebra. Just a method used 
@@ -106,19 +106,20 @@ class relation (object):
         newt.header=header(list(self.header.attributes))
         for i in self.content:
             for j in range(len(self.header.attributes)):
-                if i[j].isdigit():
+                if len(i[j])>0 and i[j].isdigit():
                     attributes[self.header.attributes[j]]=int(i[j])
-                elif rstring(i[j]).isFloat():
+                elif len(i[j])>0 and rstring(i[j]).isFloat():
                     attributes[self.header.attributes[j]]=float(i[j])
-                elif isDate(i[j]):
+                elif len(i[j])>0 and isDate(i[j]):
                     attributes[self.header.attributes[j]]=rdate(i[j])
                 else:
                     attributes[self.header.attributes[j]]=i[j]
                 
-            
-            
-            if eval(expr,attributes):
-                newt.content.append(i)
+            try:
+                if eval(expr,attributes):
+                    newt.content.append(i)
+            except Exception,e:
+                raise Exception("Failed to evaluate %s\n%s" % (expr,e.__str__()))
         return newt
     def product (self,other):
         '''Cartesian product, attributes must be different to avoid collisions
@@ -200,7 +201,7 @@ class relation (object):
         Will return an empty one if there are no common items.
         Will return None if headers are different.
         It is possible to use projection and rename to make headers match.'''
-        other=self.rearrange(other) #Rearranges attributes' order
+        other=self._rearrange_(other) #Rearranges attributes' order
         if (self.__class__!=other.__class__)or(self.header!=other.header):
             return None
         newt=relation()
@@ -218,7 +219,7 @@ class relation (object):
         Will return an empty one if the second is a superset of first.
         Will return None if headers are different.
         It is possible to use projection and rename to make headers match.'''
-        other=self.rearrange(other) #Rearranges attributes' order
+        other=self._rearrange_(other) #Rearranges attributes' order
         if (self.__class__!=other.__class__)or(self.header!=other.header):
             return None
         newt=relation()
@@ -229,7 +230,35 @@ class relation (object):
             if e not in other.content:
                 newt.content.append(list(e))
         return newt
-    
+    def division(self,other):
+        '''Division operator
+        The division is a binary operation that is written as R ÷ S. The
+        result consists of the restrictions of tuples in R to the
+        attribute names unique to R, i.e., in the header of R but not in the
+        header of S, for which it holds that all their combinations with tuples
+        in S are present in R. 
+        '''
+        
+        #d_headers are the headers from self that aren't also headers in other
+        d_headers=list(set(self.header.attributes) - set(other.header.attributes))
+        
+        
+        '''
+        Wikipedia defines the division as follows:
+        
+        a1,....,an are the d_headers
+        
+        T := πa1,...,an(R) × S
+        U := T - R
+        V := πa1,...,an(U)
+        W := πa1,...,an(R) - V
+        
+        W is the result that we want
+        '''
+        
+        t=self.projection(d_headers).product(other)
+        return self.projection(d_headers).difference(t.difference(self).projection(d_headers))
+        
     def union(self,other):
         '''Union operation. The result will contain items present in first
         and second operands.
@@ -237,7 +266,7 @@ class relation (object):
         Will not insert tuplicated items.
         Will return None if headers are different.
         It is possible to use projection and rename to make headers match.'''
-        other=self.rearrange(other) #Rearranges attributes' order
+        other=self._rearrange_(other) #Rearranges attributes' order
         if (self.__class__!=other.__class__)or(self.header!=other.header):
             return None
         newt=relation()
@@ -259,8 +288,6 @@ class relation (object):
         '''Does a left and a right outer join and returns their union.'''
         a=self.outer_right(other)
         b=self.outer_left(other)
-        print a
-        print b
         
         return a.union(b)
         
@@ -373,7 +400,7 @@ class relation (object):
     def __eq__(self,other):
         '''Returns true if the relations are the same, ignoring order of items.
         This operation is rather heavy, since it requires sorting and comparing.'''
-        other=self.rearrange(other) #Rearranges attributes' order so can compare tuples directly
+        other=self._rearrange_(other) #Rearranges attributes' order so can compare tuples directly
         if (self.__class__!=other.__class__)or(self.header!=other.header):
             return False #Both parameters must be a relation
         
@@ -549,7 +576,4 @@ class header (object):
                 if i==self.attributes[j]:
                     res.append(j)
         return res
-        
-
-if __name__=="__main__":
-  pass
\ No newline at end of file
+        
\ No newline at end of file
