@@ -2,7 +2,7 @@
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 # 
-# Relation is free software: you can redistribute it and/or modify
+# Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
@@ -30,7 +30,7 @@ class Ui_Form(object):
         self.About=None
         self.Survey=None
         self.relations={} #Dictionary for relations
-        self.undo=[] #UndoQueue for relations
+        self.undo=[] #UndoQueue for queries
         self.selectedRelation=None
     def load_query(self,*index):
         self.txtQuery.setText(self.savedQ.itemData(index[0]).toString())
@@ -44,17 +44,20 @@ class Ui_Form(object):
             
         self.savedQ.addItem(res[0],QtCore.QVariant(self.txtQuery.text()))
     def toggle_advanced(self):
+        '''Hides or shows the advanced frame'''
         if self.advancedBox.isVisible():
             self.advancedBox.hide()
         else:
             self.advancedBox.show()
         
     def undo_optimize(self):
+        '''Undoes the optimization on the query, popping one item from the undo list'''
         try:
             self.txtQuery.setText(self.undo.pop())
         except:#Nothing to restore
             pass
     def optimize(self):
+        '''Performs all the possible optimizations on the query'''
         self.undo.insert(len(self.undo),self.txtQuery.text()) #Storing the query in undo list
         
         result=optimizer.optimize_all(str(self.txtQuery.text().toUtf8()),self.relations)
@@ -62,6 +65,7 @@ class Ui_Form(object):
         
         #self.txtQuery.setText(result)
     def execute(self):
+        '''Executes the query'''
         try:
             #Converting string to utf8 and then from qstring to normal string
             query=str(self.txtQuery.text().toUtf8())
@@ -80,9 +84,11 @@ class Ui_Form(object):
             self.updateRelations() #update the list
             self.selectedRelation=result
             self.showRelation(self.selectedRelation) #Show the result in the table
-        except:
-            QtGui.QMessageBox.information(None,QtGui.QApplication.translate("Form", "Error"),QtGui.QApplication.translate("Form", "Check your query!")  )
+        except Exception, e:
+            print e
+            QtGui.QMessageBox.information(None,QtGui.QApplication.translate("Form", "Error"),"%s\n%s" % (QtGui.QApplication.translate("Form", "Check your query!"),e.__str__())  )
     def showRelation(self,rel):
+        '''Shows the selected relation into the table'''
         self.table.clear()
         
         if rel==None: #No relation to show
@@ -110,6 +116,7 @@ class Ui_Form(object):
             self.showRelation(self.selectedRelation)
             
     def showAttributes(self,*other):
+        '''Shows the attributes of the selected relation'''
         for i in other:
             rel=str(i.text().toUtf8())
             self.lstAttributes.clear()
@@ -143,6 +150,7 @@ class Ui_Form(object):
         ui = survey.Ui_Form()
         self.Survey.setUi(ui)
         ui.setupUi(self.Survey)
+        self.Survey.setDefaultValues()
       self.Survey.show()
     def showAbout(self):
         if self.About==None:
@@ -151,13 +159,22 @@ class Ui_Form(object):
             ui.setupUi(self.About)
         self.About.show()
 
-    def loadRelation(self):
+    def loadRelation(self,filename=None,name=None):
+        '''Loads a relation. Without parameters it will ask the user which relation to load,
+        otherwise it will load filename, giving it name.
+        It shouldn't be called giving filename but not giving name.'''
         #Asking for file to load
-        filename = QtGui.QFileDialog.getOpenFileName(None,QtGui.QApplication.translate("Form", "Load Relation"),"",QtGui.QApplication.translate("Form", "Relations (*.csv);;Old Relations (*.tlb);;Text Files (*.txt);;All Files (*)"))
+        if filename==None:
+            filename = QtGui.QFileDialog.getOpenFileName(None,QtGui.QApplication.translate("Form", "Load Relation"),"",QtGui.QApplication.translate("Form", "Relations (*.csv);;Old Relations (*.tlb);;Text Files (*.txt);;All Files (*)"))
+                
+            #Default relation's name
+            f=str(filename.toUtf8()).split('/') #Split the full path
+            defname=f[len(f)-1].lower() #Takes only the lowercase filename
+            
+        else:
+            f=filename.split('/') #Split the full path
+            defname=f[len(f)-1].lower() #Takes only the lowercase filename
         
-        #Default relation's name
-        f=str(filename.toUtf8()).split('/') #Split the full path
-        defname=f[len(f)-1].lower() #Takes only the lowercase filename
         if len(defname)==0:
             return
         use_csv=True
@@ -166,16 +183,20 @@ class Ui_Form(object):
             defname=defname[:-4]
             use_csv=False #Old format, not using csv
             
-        
         if (defname.endswith(".csv")): #removes the extension
             defname=defname[:-4]
         
-        res=QtGui.QInputDialog.getText(self.Form, QtGui.QApplication.translate("Form", "New relation"),QtGui.QApplication.translate("Form", "Insert the name for the new relation"),
-        QtGui.QLineEdit.Normal,defname)
-        if res[1]==False:
-            return
-        
-        self.relations[str(res[0].toUtf8())]=relation.relation(filename,use_csv)
+        if name==None: #Prompt dialog to insert name for the relation
+            res=QtGui.QInputDialog.getText(self.Form, QtGui.QApplication.translate("Form", "New relation"),QtGui.QApplication.translate("Form", "Insert the name for the new relation"),
+            QtGui.QLineEdit.Normal,defname)
+            if res[1]==False or len(res[0])==0:
+                return
+            
+            #Patch provided by Angelo 'Havoc' Puglisi
+            self.relations[str(res[0].toUtf8())]=relation.relation(str(filename.toUtf8()),use_csv)            
+        else: #name was decided by caller
+            self.relations[name]=relation.relation(filename,use_csv)
+                
         self.updateRelations()
     def insertTuple(self):
         '''Shows an input dialog and inserts the inserted tuple into the selected relation'''
@@ -215,6 +236,9 @@ class Ui_Form(object):
     def addIntersection(self):
         self.txtQuery.insert(u"ᑎ")
         self.txtQuery.setFocus()
+    def addDivision(self):
+        self.txtQuery.insert(u"÷")
+        self.txtQuery.setFocus()
     def addOLeft(self):
         self.txtQuery.insert(u"ᐅLEFTᐊ")
         self.txtQuery.setFocus()
@@ -239,6 +263,7 @@ class Ui_Form(object):
     def addArrow(self):
         self.txtQuery.insert(u"➡")
         self.txtQuery.setFocus()
+    
         
     def setupUi(self, Form):
         self.Form=Form
@@ -284,6 +309,10 @@ class Ui_Form(object):
         self.cmdIntersection.setMaximumSize(QtCore.QSize(16777215,16777215))
         self.cmdIntersection.setObjectName("cmdIntersection")
         self.verticalLayout.addWidget(self.cmdIntersection)
+        self.cmdDivision = QtGui.QPushButton(self.groupBox)
+        self.cmdDivision.setMaximumSize(QtCore.QSize(16777215,16777215))
+        self.cmdDivision.setObjectName("cmdDivision")
+        self.verticalLayout.addWidget(self.cmdDivision)
         self.cmdJoin = QtGui.QPushButton(self.groupBox)
         self.cmdJoin.setMaximumSize(QtCore.QSize(16777215,16777215))
         self.cmdJoin.setObjectName("cmdJoin")
@@ -462,6 +491,7 @@ class Ui_Form(object):
         QtCore.QObject.connect(self.cmdDifference,QtCore.SIGNAL("clicked()"),self.addDifference)
         QtCore.QObject.connect(self.cmdUnion,QtCore.SIGNAL("clicked()"),self.addUnion)
         QtCore.QObject.connect(self.cmdIntersection,QtCore.SIGNAL("clicked()"),self.addIntersection)
+        QtCore.QObject.connect(self.cmdDivision,QtCore.SIGNAL("clicked()"),self.addDivision)
         QtCore.QObject.connect(self.cmdOuterLeft,QtCore.SIGNAL("clicked()"),self.addOLeft)
         QtCore.QObject.connect(self.cmdJoin,QtCore.SIGNAL("clicked()"),self.addJoin)
         QtCore.QObject.connect(self.cmdOuterRight,QtCore.SIGNAL("clicked()"),self.addORight)
@@ -521,6 +551,8 @@ class Ui_Form(object):
         self.cmdUnion.setText(QtGui.QApplication.translate("Form", "ᑌ", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdIntersection.setToolTip(QtGui.QApplication.translate("Form", "Intersection operator", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdIntersection.setText(QtGui.QApplication.translate("Form", "ᑎ", None, QtGui.QApplication.UnicodeUTF8))
+        self.cmdDivision.setToolTip(QtGui.QApplication.translate("Form", "Division operator", None, QtGui.QApplication.UnicodeUTF8))
+        self.cmdDivision.setText(QtGui.QApplication.translate("Form", "÷", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdJoin.setToolTip(QtGui.QApplication.translate("Form", "Natural join operator", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdJoin.setText(QtGui.QApplication.translate("Form", "ᐅᐊ", None, QtGui.QApplication.UnicodeUTF8))
         self.cmdOuterLeft.setToolTip(QtGui.QApplication.translate("Form", "Outer join left operator", None, QtGui.QApplication.UnicodeUTF8))
